;jm1A, file: f877_jmonA.inc
;==============================================================================
;JACK'S MONITOR, version 1A.0
;bootloader and terminal communication program for the Microchip PIC16f87x
;    Copyright (C) 2003-Jack Dollhausen (jackdoll@verizon.net)
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
;==============================================================================
;	nolist
	list
;
; this header file defines configurations, registers, and other useful equates
; for the pic16f877 microcontroller - names should match the data sheets and gpasm
;
; note that the processor must be selected one of these ways:
;       1. command line switch:
;               mpasm myfile.asm /pic16f877
;       2. list directive in the source file
;               list   p=pic16f877
;==============================================================================
;monitor defined variables, -all bank3 to keep low banks open for user programs
;handy for developing code for 'f84 in the 'f87x
;user can define this as badram to protect it if necessary
                cblock  0190h
;
wcount:1	;word count from hex file line
checksum:1	;accumulated from incoming data during load
wctr:1		;count down for timeout, linebytes
hexbyte:1	;translated from ascii usart input
mtmp:1		;general purpose storage
xbuff:2		;hexword store
line_buff:010h  ;line data buffer for flash write (16 words,max line)
;
                endc
	nolist
;==========================================================================
;monitor subroutines can be called from user code
;these can be cut and pasted to define the subroutines in user code
;monitor subroutines.......(n)=returned RAM bank
;
;monitor	()jump into monitor, hard reset
;runmon		()jump into monitor
;eeread		(2)eeread returns byte addressed in eeadr
;eewrite	(3)eewrite writes the byte to eeprom memory
				; at the current eeadr
;flashread	(3)read a location in flash program memory
				;,or in eeprom data
                      		;address is in eeadrh and eeadr
                      		;data returns in eedath and eedata
;getdigit	(0)get usart byte and convert to hex digit
				; in low nibble of w
;gethexbyte	(3)two ascii digits from the usart-
				;convert to a byte in w
;gethexword	(3)two hex bytes returned in addrfh,addrfl
;getkey		(0)getkey checks input for esc,goes to runmon
				;,or returns ascii key
;sendbyte	(0) byte in w as two hex char to usart
;sendcrlf	(0)send a cr/lf sequence
;serialsetup	(0)initialize the usart for tx,rx
				; at default baud rate
;serrx		(0)read a byte from usart, return in w
;sertx		(0)transmit byte from w register
;sendstring     (2)unpacks one ascii line to the usart
			;strings are format: <label> da "string\0" (nul term)
			;must enter with eeadr,eeadrh pointing to a string
			;macro mssg calls this routine
eeread          equ  01f77h
eewrite         equ  01f69h
getdata         equ  01e73h
getdigit        equ  01fafh
gethexbyte      equ  01fa4h
gethexword      equ  01f9fh
getkey          equ  01fb7h
monitor         equ  01d00h
runmon          equ  01d23h
runnn           equ  01ee8h
sendbyte        equ  01fbfh
sendcrlf        equ  01f64h
sendstring      equ  01fe4h
serialsetup     equ  01f89h
serrx           equ  01f93h
sertx           equ  01f99h

;==========================================================================
;baud rate constant with 20MHz clock...
baud_rate      equ     040h	;19200 d'64'(subroutine serialsetup)
	;baud_9600       equ     081h	;d'129'
;==============================================================================
;       register and bit names
;==============================================================================
w	equ     0000h
f	equ     0001h
;special function registers----------------------------------------------------
indf	equ     0000h
tmr0	equ     0001h
pcl	equ     0002h
status	equ     0003h
fsr	equ     0004h
porta	equ     0005h
portb	equ     0006h
portc	equ     0007h
portd	equ     0008h
porte	equ     0009h
pclath	equ     000ah
intcon	equ     000bh
pir1	equ     000ch
pir2	equ     000dh
tmr1l	equ     000eh
tmr1h	equ     000fh
t1con	equ     0010h
tmr2	equ     0011h
t2con	equ     0012h
sspbuf	equ     0013h
sspcon	equ     0014h
ccpr1l	equ     0015h
ccpr1h	equ     0016h
ccp1con	equ     0017h
rcsta	equ     0018h
txreg	equ     0019h
rcreg	equ     001ah
ccpr2l	equ     001bh
ccpr2h	equ     001ch
ccp2con	equ     001dh
adresh	equ     001eh
adcon0	equ     001fh
option_reg  equ     0081h
trisa	equ     0085h
trisb	equ     0086h
trisc	equ     0087h
trisd	equ     0088h
trise	equ     0089h
pie1	equ     008ch
pie2	equ     008dh
pcon	equ     008eh
sspcon2	equ     0091h
pr2	equ     0092h
sspadd	equ     0093h
sspstat	equ     0094h
txsta	equ     0098h
spbrg	equ     0099h
adresl	equ     009eh
adcon1	equ     009fh
eedata	equ     010ch
eeadr	equ     010dh
eedath	equ     010eh
eeadrh	equ     010fh
eecon1	equ     018ch
eecon2	equ     018dh
;serial bits ------------------------------------------------------------------
rcif	equ     0005h
eepgd	equ     0007h
wr	equ     0001h
wren	equ     0002h
rd	equ     0000h
brgh	equ     0002h
txen	equ     0005h
cren	equ     0004h
spen	equ     0007h
txif	equ     0004h
;status bits ------------------------------------------------------------------
irp	equ     0007h
rp1	equ     0006h
rp0	equ     0005h
not_to	equ     0004h
not_pd	equ     0003h
z	equ     0002h
dc	equ     0001h
c	equ     0000h
;intcon bits ------------------------------------------------------------------
gie	equ     0007h
peie	equ     0006h
t0ie	equ     0005h
inte	equ     0004h
rbie	equ     0003h
t0if	equ     0002h
intf	equ     0001h
rbif	equ     0000h
;===================================================================
;       ram define
;===================================================================
        __maxram 01ffh
        __badram 08fh-090h, 095h-097h, 09ah-09dh
        __badram 0105h, 0107h-0109h
        __badram 0185h, 0187h-0189h, 018eh-018fh
;===================================================================;
;       config bits
;===================================================================
_cp_all			equ     0fcfh
_cp_half		equ     01fdfh
_cp_upper_256		equ     02fefh
_cp_off                      equ     03fffh
_debug_on                    equ     037ffh
_debug_off                   equ     03fffh
_wrt_enable_on               equ     03fffh
_wrt_enable_off              equ     03dffh
_cpd_on                      equ     03effh
_cpd_off                     equ     03fffh
_lvp_on                      equ     03fffh
_lvp_off                     equ     03f7fh
_boden_on                    equ     03fffh
_boden_off                   equ     03fbfh
_pwrte_off                   equ     03fffh
_pwrte_on                    equ     03ff7h
_wdt_on                      equ     03fffh
_wdt_off                     equ     03ffbh
_lp_osc                      equ     03ffch
_xt_osc                      equ     03ffdh
_hs_osc                      equ     03ffeh
_rc_osc                      equ     03fffh
;===================================================================
;utility macros.....
;macros to select the register bank
;can be optimized when only one status bit changes
bank0 macro ;macro to select data ram bank 0
 	bcf status, rp0
	bcf status, rp1
	endm
	;
bank1 macro ;macro to select data ram bank 1
	bsf status, rp0
	bcf status, rp1
	endm
	;
bank2 macro ;macro to select data ram bank 2
	bcf status, rp0
	bsf status, rp1
	endm
	;
bank3 macro ;macro to select data ram bank 3
	bsf status, rp0
	bsf status, rp1
	endm
	;
show macro label ;send the byte @ label to usart
	banksel label
	movfw label
	lcall sendbyte
	pagesel $
	endm
	;
;##################################################
;mssg sets up and sends the string sname to terminal
;see subs.03 (sendstring), for format
mssg macro sname
	banksel eeadr
	movlw high sname
	movwf eeadrh
	movlw low sname
	movwf eeadr
	call sendstring
	endm
;##################################################

        list
