	list
;jmonV1A.0, file: jm1A.asm - main code
;this assembles with gpasm (gputils 0.13.3)
;==============================================================================
;JACK'S MONITOR, version 1A.0
;bootloader and terminal communication program for the Microchip(R) PIC16f87x 
;    Copyright (C) 2001-6, Jack Dollhausen (jackdoll@verizon.net)
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
;==============================================================================
;4/8/2006, V1A.0 revised to write to flash in four word blocks to accomodate
;microchip's 'A' series pic16f87x chips
;==============================================================================
	list
;main code
;limited to 768 bytes of flash memory at $1d00-$1fff,
;and 3bytes at reset vector $0000.
;==============================================================================
        list p=16f877,n=0
        errorlevel -302,-306
        include <f877_jmonA.inc> ;def header
        ;
;the programmer does config, -this is therefore the
;default config for user code

        __config _boden_off & _cp_off & _pwrte_on & _wdt_off & _wrt_enable_on & _hs_osc & _debug_off & _cpd_off & _lvp_off
;__________________________________________________________
	org	000h ;reset vector
	;
	movlw   high main ;reset vector	always jumps to monitor
	movwf   pclath
	b    main ;go to monitor main code,wait for 4sec. for a key
			;or jump to location 0020
;__________________________________________________________
;int vector @ 00004h
;user code  @ 00020h - 01bffh
;__________________________________________________________
	org	01d00h ;768 bytes for resident monitor code(1d00-1fff)
;
monitor	;reset entry point
main
	pagesel $
	call serialsetup
	mssg foursec ;send version message
	;
	bsf status,rp0   ;bank3	;option reg, and all monitor variables
	movlw 	b'00000111'	;tmr0 prescaler, 1:256
	movwf option_reg
	movlw 0ffh ;time counter to wait for key after reset
			;adjust this delay proportional to clock speed
	movwf wctr ;full counter gives about four seconds @ 20MHz
			;

timeout		;waits for a key or timeout
	bank0
	btfsc pir1,rcif ;poll usart receive bit
	b runmon 	;if key sent
	btfss intcon,t0if	;wait for timer0 to timeout
	b timeout  ;loop
	bcf intcon,t0if
	bank3 	;wctr
	decfsz wctr,f
	b timeout   ;unless time ends with no key input
	;
runcode
	mssg runnn  ;run code message
	;
	clrf pclath
	b 020h   ;run user code
;__________________________________________________________
;enter here to skip delay and jump
;
runmon	;start running the monitor, show menu and get choice
	pagesel $
	call sendcrlf
	call prompt ;
;
choose   ;choose action, one of 16 ops
	movlw high monitor
	movwf pclath
	call getkey  ;get choice
	andlw b'0000111' ;mask 3 lsb for 8 ops
	btfss status,z ;avoid loop of adding zero to the pc
	addwf pcl,f  ;compute the jump
	b peekflash ;function 0...
	b peekdata	;1
	b peekee	;2
	b loader 	;3-load a new hex image into program memory
	b runcode	;4-run user program(if it's there)
	b dumpee	;5-dump eeprom to usart
	b togint 	;6-toggle gie (interrupt)
	b runmain	;7-(v3.1) run at main,skip init
	b runmon
;__________________________________________________________
;peekflash displays program memory, allows poke, scrollup/dn
peekflash   ;poke removed in v1A.0
	mssg prgmem
	call gethexbyte
	bcf status,rp0  ;bank2  ;sel eeadr
	andlw 01fh ;limit high address
	movwf eeadrh ;for flashread
	call gethexbyte
	bcf status,rp0  ;bank2  ;sel eeadr
	movwf eeadr
pfloop			;prints a pgm line to scrn
	call sendcrlf
	bank2  ;sel eeadr
	movfw eeadrh
	call sendbyte
	bank2
	movfw eeadr
	call sendbyte ;send address word to terminal
	call flashread ;returns bank3
	;
	movlw ' '
	call sertx ;returns bank0
	bank2
	movfw eedath
	call sendbyte
	bank2
	movf eedata,w
	call sendbyte ;send program word to terminal
	;
pfl_com
	call getkey ;location displayed, wait for key
	xorlw '0'  ;scroll up
	btfsc status,z  ;
	b incflsh
	xorlw '0'  ;restore key
	xorlw '1'  ;scroll down
	btfss status,z  ;
	b pfl_com ;loop for a valid key
	;
decflsh		;scroll down
	bank2
	movlw 001h
	subwf eeadr,f  ;decrement low address
	btfss status,c
	decf eeadrh,f
	b pfloop
incflsh		;scroll up
	bank2
	incfsz eeadr,f
	b pfloop
	incf eeadrh,f ;if low rolls to high
	b pfloop
	;
;__________________________________________________________
;peekdata displays ram, allows poke, scrollup/dn
;
peekdata
	;bcf intcon,gie  ;leave the int alone for on-the-fly
	mssg datmem ;RAM @>
	call gethexword  ;pointer to ram
;
dloop
	bank3
	movf xbuff+1,f ;if not zero
	btfsc status,z ;make it one
	b dloopa ;else clear bank bit
	movlw 001h  ;highaddr max 0000-01ff
	movwf xbuff+1
	bsf status,irp ;(baniksel 2,3)
	b dloopb
dloopa	bcf status,irp ;(baniksel 0,1)
;
dloopb	call sendcrlf
	bank3
	movfw xbuff+1 ;show address, high
	call sendbyte
	bank3
	movfw xbuff ;show address, low
	movwf fsr ;indirect address in index
	call sendbyte
	movlw '-'
	call sertx ;returns bank0
	movfw indf ;get data byte to display
	call sendbyte ;show it and loop or poke or escape
dlpx	;
	call getkey ;will jump out to runmon if 'esc'
	xorlw 'p' ;poke this data address
	btfsc status,z
	b dpoke
	;
	xorlw 'p'  ;restore key
	xorlw '0'  ;scroll up
	btfsc status,z  ;
	b incdta  ;increment address
	;
	xorlw '0'  ;restore key
	xorlw '1'  ;scroll down
	btfsc status,z  ;
	b decdta ;decrement address
	;
	xorlw '1'  ;restore key
	xorlw 'i'  ;increment/decrement value
	btfsc status,z
	b ival
	xorlw 'i'  ;restore
	xorlw 'd'  ;
	btfsc status,z
	b dval
	xorlw 'd'  ;restore
	xorlw 'b'  ;bit toggle b76543210
	btfsc status,z
	b bbng
	xorlw 'b'
	xorlw 'f'
	btfss status,z
	b dlpx ;escape this loop on wild key
		;'f' fills hh places, with xx
	mssg filln
	;
tagn
	call gethexbyte ;how many?
	andlw 01fh  ;limit to 5bits for damage control
		;*caution* -this will wrap into sfr's
	btfsc 3,z ;no max count
	b tagn ;loop for valid count
	;
	movwf mtmp  ;counter
	movlw '='
	call sertx
	call gethexbyte  ;with what?
fruzg
	movwf indf
	incf fsr,f
	decfsz mtmp,f
	b fruzg ;for n bytes
	;
	movfw fsr
	movwf xbuff ;advance pointer past load
	b dloop
;
bbng   ;toggle bits in variable
	call sendcrlf
	mssg bittg
	movfw indf
	call sendbyte
bbngx
	movlw 01h
	movwf mtmp ;bit marker
	call serrx
	xorlw 01bh ;esc
	btfsc 3,z
	b dloopb
	xorlw 01bh ;restore
	;
	andlw 07h ;accept 3lsb bits
	btfsc 3,z
	b togg
	movwf wctr  ;loop counter
	;
	bcf 3,c
	rlf mtmp,f ;set bit marker
	decfsz wctr,f
	b $-2
togg
	movfw mtmp
	xorwf indf,f
	call shinc
	b bbngx  ;more bits or escape

dval   ;decrement the variable
	decf indf,f
	call shinc
	b dlpx  ;idloop  ;next i/d

ival   ;increment the variable
	incf indf,f
	call shinc
	b dlpx  ;idloop  ;next i/d

shinc  ;show the i/d realtime
	movlw 008h ;bkspc chr
	call sertx
	call sertx ;cover old value
	movfw indf
	call sendbyte ;new value
	return
	;
;inc/dec current address..........
decdta
	bank3
	movlw 001h
	subwf xbuff,f  ;decrement low address
	btfss status,c
	decf xbuff+1,f ;if roll
	b dloop
incdta
	bank3
	incfsz xbuff,f
	b dloop
	incf xbuff+1,f	;data address rolls over
	b dloop
	;
dpoke
	mssg ppoke
	call gethexbyte
	movwf indf
	b dloop
;__________________________________________________________
;dumpee does a hex dump of the 256 eeprom bytes
;
dumpee
	bcf intcon,gie  ;dumpee shuts down interrupts
	mssg edmp
	bsf status,rp1  ;bank2
	clrf eeadr ;start at address 0
reelx
	call sendcrlf
	bank2
	movfw eeadr
	call sendbyte
	movlw ':'
	call sertx
reeloop
	call eeread
	call sendbyte
	movlw ' '
	call sertx
	bank2
	incfsz eeadr,f
	b linechk
	b runmon  ;finished
linechk
	movlw 00fh ;mask low bits for newline
	andwf eeadr,w
	btfsc status,z
	b reelx ;newline
	b reeloop
;__________________________________________________________
togint  ;toggle interrupt enable intcon,7
	mssg intrpt
	movlw 080h
	xorwf intcon,f
	movfw intcon
	call sendbyte
	b runmon
;__________________________________________________________
;peekee displays and pokes eeprom data memory, shows consecutive 
;locations on each keypress except esc key which jumps back to menu
;
peekee
	mssg eeprom
	call gethexbyte
	bcf status,rp0  ;bank2
	movwf eeadr  ;ee is addressed
eepromrd
	call sendcrlf
	bsf 3,rp1  ;bank2
	movfw eeadr
	call sendbyte
	movlw '-'
	call sertx
	call eeread
	call sendbyte
	;
	call getkey
	xorlw 'p' ;poke this address
	btfss status,z
	b eeprx  ;any other key increments the address and loops
	movlw '='
	call sertx
	call gethexbyte
	bcf status,rp0  ;bank2
	movwf eedata
	call eewrite
	b eepromrd ;and loop without increment
eeprx
	bank2	;sel eeadr
	incf eeadr,f
	b eepromrd
	;
;__________________________________________________________
;loader receives a hex file, and programs flash memory
;hexfile in intel hx32 something
;02/15/05
;this version of the monitor skips the first line of the
;file, if it has a non-valid record type
;monitor only accepts 00=data, 01=eof, but must accomodate
;the first line that gpasm sticks on the hex file
;
loader
	bcf intcon,gie  ;interrupts off
	mssg loadr
;
ldloop ;parse and write data for one hex line.........
;
        call getkey  ;allows escape to monitor
        xorlw ':' ;if it's a colon, a new line is starting(colon=03ah)
        btfss status,z
        b ldloop	;wait for valid line indicator(:),or esc in getkey
;--------------------------------------------------------
;parse line header for wordcount, bytecount, address and type
;on-the-fly until eof (type 01)line is received
;or if no eof,wait for next colon or esc key from kbd
;
gethdr
  ;process first eight bytes of line
  	bank3
        clrf checksum ;line checksum is zero to start cumulative count
	;
;first byte is the line bytecount
        call getbin  ;adds to line checksum
        movwf wcount ;number of words in line
	;
;2nd byte is upper half program address
        call getbin
        movwf mtmp  ;(eeadrh)high byte of address to write
		;store for octave shift, (words and bytes)
;3rd byte is low half of address
        call getbin  ;returns bank3
	bcf 3,rp0   ;bank2
        movwf eeadr  ;low addr
	;
	bcf status,c ;clear carry bit
	bsf 3,rp0  ;bank3
        rrf mtmp,w ;/2 to get word address in real flash
	bcf 3,rp0  ;bank2
	movwf eeadrh
        rrf eeadr,f ;actual addr is 1/2
        ;
;protect monitor code.......
	movlw 01dh  ;high addr of resident code
	subwf eeadrh,w
	btfsc status,c ;if greater or equal,c=1
	b err_1 ;protect code error
	;
;4th byte is record type,-only 00,01 are recognized
        call getbin  ;returns bank3
        andlw 0ffh  ;tt=00 is data record
        btfsc status,z  ;
	b getdata ;fill line buffer
	;
	xorlw 01h  ;not 0,so is it 01?
	btfsc 3,z
	b filedone	;tt=01 is eof
        ;
	;mssg gpasm  ;print skipped line mssg
	;b ldloop
	
	b linedone ;and skip this weird line
	;
;------------------------------------------------
;5th byte is data
getdata
	bankisel line_buff  ;bank3 index
	movlw low line_buff ;set index
	movwf fsr
	movfw wcount	;# of bytes in file line
	movwf wctr
	;
bld	;loop to fill buffer
	call getbin ;get data byte
	movwf indf  ;into linebuffer
	incf fsr,f ;adv ptr
	decfsz wctr,f  ;bytecount down
	b bld
	;
;last byte is checksum
        call getbin  ;get checksum
        movf checksum,w  ;should be=zero
        btfss status,z
        b err_2 ;flag checksum fault
        ;
padbuf	;if line has trailing space, fill with blanks
	btfsc wcount,4 ;full line?
	b fline
	incf wcount,f
	movlw 0ffh
	movwf indf
	incf fsr,f
	b padbuf
	;
;------------------------------------------------
;
fline  ;write 8 words from buffer to flash (32digits,16bytes)
	;linebuffer holds flashdata words, wcount is bytecount
	
;
	movlw low line_buff ;reset index
	movwf fsr
	movlw 02h ;number of 4byte blocks, loop counter
	movwf mtmp  ;four block counter
	;
	
blocklp	;flash four words
	bank2
	movfw indf ;get a byte
	movwf eedata ;to write flash lobyte
	incf fsr,f
	movfw indf ;
	movwf eedath ;flash data hibyte
	incf fsr,f
	;
	bsf 3,rp0  ;bank3
	bsf eecon1,eepgd  ;select flash,not ee
	bsf eecon1,wren  ;allow write
   ;...................................
	movlw   055h  ;(microchip says this sequence is required)
	movwf   eecon2
	movlw   0aah
	movwf   eecon2
	bsf     eecon1,wr ;begin write
	nop
	nop
   ;...................................
	bcf eecon1,wren ;take no chances on spurious writes to ee
	bcf 3,rp0 ;bank2
	incf eeadr,f  ;for four words
	movlw 03h
	andwf eeadr,w
	bnz blocklp
	;
	bsf 3,rp0  ;bank3
	decfsz mtmp,f ;countdown two blocks=4wds
	b blocklp
;
linedone
        movlw   '*'  ;line has been programmed so
        call sertx ;transmit success
        b ldloop  ;go get next line hex file
;
filedone
	movlw   '!'  ;
        b erxnx ;transmit finished
;__________________________________________________________
;error breaks
;
err_2 ;checksum fault
	mssg err02
	b erxnn
err_1 ;address in resident code (>01d00h)
	mssg err01
erxnn
	call sertx
	movlw '~'
	call sertx
	movlw 'X'
erxnx	call sertx
;more incoming data is ignored until timeout...
	b tt_ox
tt_out
	bank0
	btfss pir1,rcif
	b d_delay
	movf rcreg,w   ;data in w,(clr pir1)
tt_ox	bank3
	clrf xbuff ;keep delay going
	clrf xbuff+1 ;until garbage is done
	;
d_delay  ;16bit countdown
	bank3
	decfsz xbuff,f ;monitor variable
	b tt_out
	decfsz xbuff+1,f
	b tt_out
	b runcode  ;or b monitor?
;__________________________________________________________
runmain
	mssg runm
	movlw 018h  ;diagnostics vector 01800h
	movwf pclath
	b 0h  ;vector jump to "main" in user
		;diag must define the vector jump
		;at address 01800h - this avoids init
		;command 04h always jumps to addr 0020h,
		;re-initializing variables
;==============================================================================
;__________________________________________________________
;message strings
foursec	da "Jack's Monitor V1A.0 \x0d\x0a\0"

runnn	da "Run code: 0020\x0d\x0a\0"

eeprom	da "Eeprom @>\0"

ppoke	da " Poke>\0"

edmp	da "Eeprom dump\x0d\x0a\0"

datmem	da "Ram @>\0"

prgmem	da "Flash @>\0"

loadr	da "ALoad hex\x0d\x0a\0"

intrpt	da "Toggle GIE \0"

runm	da "b main\x0d\x0a\0"

bittg	da "Toggle bit#> \0"

gpasm	da "skip line\x0d\x0a\0"

filln	da " Fill>\0"

err02	da "checksum\0"

err01	da "protect\0"

endmsgs
	da "(C)2001-6,Jack Dollhausen(GnuPL)"
	nop
	nop
;=================================================
	org 01f60h
	include <subs1A.inc>
	;page3.............01800h,(03000h in hex file)
;monitor and interactive diagnostics
;monitor inner sanctum starts at 01d00h,768 words
;---------------------------------------------
	org 01800h	;vector for user defined jump to code
		;a jump to $ will start at main in current program
		;rvector in monitor...
	;pagesel main  ;on command 7
	;b main ;and runs at main without initializing
;=========================================================
;8K flash memory page map for pic16f87x with monitor installed
;page0............00000h
			;00000h-00003h reserved monitor jump vector
			;00004h-0001fh reserved interrupt call
			;00020h- user code
;page1............00800h, (01000h in hex file)
;page2............01000h, (02000h in hex file)
;page3............01800h, (03000h in hex file)
			;01d00h-01fff-reserved for resident monitor
;=========================================================
                end

