	list
;jm1A, file: subs1A.inc - subroutines called by jm1A.asm
;==============================================================================
;JACK'S MONITOR, version 1.0
;bootloader and terminal communication program for the Microchip PIC16f87x 
;    Copyright (C) 2003-Jack Dollhausen (jackdoll@verizon.net)
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
;==============================================================================
;flash program memory r/w routines
;==============================================================================
	list
;strings and messages
;prompt
;sends a newline & prompt
prompt
	call    sendcrlf
        movlw   '>'             ;prompt
        call    sertx
        return	;bank0
;__________________________________________________________
;sendcrlf
;send a cr/lf sequence
sendcrlf
        movlw   00dh
        call    sertx
        movlw   00ah
        call    sertx
        return	;bank0
;__________________________________________________________
;eewrite writes eedata to eeprom memory at the current eeadr
;
eewrite
	banksel eecon1
	bcf eecon1,eepgd ;eeprom memory,not program flash(bit7)
	bsf eecon1,wren	;enable writes
	bcf intcon,gie
	;.................microchip says:
	movlw 055h ;
	movwf eecon2
	movlw 0aah
	movwf eecon2
	;.................
	bsf eecon1,wr	;initiate write
	btfsc eecon1,wr ;ee int flag write done
	goto $-1
	bsf intcon,gie
	return  ;bank3
;__________________________________________________________
;eeread	returns byte addressed in eeadr
;
eeread
	bank3
	bcf eecon1,eepgd ;data, not prgm memory
	bsf eecon1,rd  ;read eeprom
	bcf status,rp0 ;bank2
	movfw eedata
	return  ;bank2
;__________________________________________________________
;flashread
;read a location in flash program memory,or in eeprom data
;address is in eeadrh and eeadr
;data returns in eedath and eedata
;
flashread
	banksel eeadrh
	movlw   01fh  ;keep address within range
	andwf   eeadrh,f
	bsf status,rp0 ;banksel eecon1 bank3
	movlw   080h ;enable reads from program flash
	movwf   eecon1
	bsf     eecon1,rd ;read from flash
	nop
	nop  		;wait while reading
	return ;returns with bank3
;==============================================================================
;usart elemental routines
;==============================================================================
;serialsetup
;initialize the usart for tx,rx at default baud rate
;
serialsetup
	bank1  ;sel spbrg
	movlw baud_rate  ;set baud rate
	movwf spbrg
	bsf txsta,brgh  ;baud rate high
	bsf txsta,txen ;enable tx
	bcf 3,rp0  ;bank0   ;rcsta
	bsf rcsta,cren  ;enable rx
	bsf rcsta,spen ;enable serial port
	return 	;returns with bank0
;__________________________________________________________
;serialreceive
;read a byte from usart, return in w
;
serrx
	bank0
	btfss pir1,rcif
	goto $-1  ;wait for chr
	movf rcreg,w   ;data in w
	return   ;returns with  bank0
;__________________________________________________________
;sertx
;transmit byte from w register
;
sertx
	banksel pir1 ;bank0
	btfss pir1,txif   ;check that buffer is empty
	goto $-1 ;loop until ready
	movwf txreg  ;transmit byte
	return ;returns with bank0
;==============================================================================
;usart send/receive string and message routines
;==============================================================================
;gethexword
;two hex bytes returned in xbuff+1,xbuff
;
gethexword
	call gethexbyte
	movwf xbuff+1 ;hibyte
	call gethexbyte
	movwf xbuff
	return ;bank3
;__________________________________________________________
;gethexbyte
;two ascii digits from the usart- convert to a byte in w
;
gethexbyte
        call getdigit
	bank3
        movwf hexbyte         ;save nibble
        swapf hexbyte,f       ;move nibble to high position
        ;
        call getdigit
	bank3
        iorwf hexbyte,f       ;add low nibble to high nibble
        movf hexbyte,w       ;put result in w reg
        return ;bank3,returns data byte in w and hexbyte
;__________________________________________________________
;gethexdigit
;get usart byte and convert to hex digit in low nibble of w
;
getdigit
	call serrx   ;get new byte from serial port
	call sertx  ;echo the chr
        addlw 0bfh	;add -'A' to ascii
        btfss status,c  ;check if positive
        addlw 007h  ;if not, add 17 ('0' to '9')
        addlw 00ah     ;or add 10 ('a' to 'f')
        andlw 00fh	;in case of upper case alpha from keyboard
        return ;bank0
        	;nibble from character in 4lsb of w register
;__________________________________________________________
;getkey checks input for esc,goes to runmon,or returns ascii key
;
getkey
	call serrx
	xorlw 01bh
	btfss status,z
	goto exkx
	movlw low runmon
	movwf pcl ;back to runmon
exkx	xorlw 01bh ;restore key
	return	;bank0
;__________________________________________________________
;sendbyte
;send the byte in w as two hex char to usart
;
sendbyte
	bank3
        movwf mtmp ; save
        swapf mtmp,w   ; move top nibble to w
        call outnyb   ; send it
	bank3
        movf mtmp,w  ; move low nibble to w
        call outnyb ; send and return
        return		;bank0
;__________________________________________________________
;fast getbyte from usart for loader, -no echo
;adds to checkdsum
getbin
        call getn
	bank3
        movwf hexbyte   ;save nibble
        swapf hexbyte,f   ;move nibble to high position
        ;
        call getn
	bank3
        iorwf hexbyte,f  ;add low nibble to high nibble
        movf hexbyte,w   ;put result in w reg
	addwf checksum,f ;always adds to checksum
        return ;bank3,returns data byte in w and hexbyte
getn
	call serrx   ;get new chr byte from serial port
        addlw 0bfh  ;add -'A' to ascii
        btfss status,c  ;check if positive
        addlw 007h  ;if not, add 17 ('0' to '9')
        addlw 00ah  ;or add 10 ('a' to 'f')
        andlw 00fh  ;in case of upper case alpha from keyboard
        return ;from bank0
;__________________________________________________________
;outnyb
;send low nibble of w as ascii hex
;
outnyb
        andlw 00fh   ;mask low nibble
        addlw 0f6h  ;add -10 to test
        btfsc status,c  ;skip if decimal
        addlw 007h   ;alpha chr
        addlw 00ah  ; restore value
        addlw '0'  ;add 030h to make it ascii
        call sertx  ; and send it
        return	;bank0
;__________________________________________________________
;sendstring
;unpacks and shoots one ascii line to the usart
;strings are format: <label>	da "string\0" (nul terminates)
;must enter with eeadr,eeadrh pointing to a string in flash pgm memory
;
sendstring
	call flashread ;returns bank3
	bcf status,rp0  ;banksel eedata
	rlf eedata,w  ;unpack ascii
	rlf eedath,w ;get ascii byte
	andlw 07fh	;limit seven lsbits
	btfsc status,z
	return	;escape if line end (0),in bank3
	call sertx
	bsf status,rp1  ;banksel eedata
	movfw eedata	;get next chr
	andlw 07fh	;mask seven lsbits
	btfsc status,z
	return	;escape if line end (0),in bank3
	call sertx
	bsf status,rp1  ;
	banksel eeadr
	incf eeadr,f
	btfsc status,z
	incf eeadrh,f
	goto sendstring ;returns bank2 from all messages
;==============================================================================
;end of subroutines include file
